<?php
/**
 * SkeletonCart
 * (c) 2017 Juxtaposition Inc. All Rights Reserved.
 * (c) 2017 Curlybracket, Inc. All Rights Reserved.
 * This code cannot be redistributed without permission from the copyright holder.
 * For more information, consult your SkeletonCart license.
 **/

namespace WPDBConnector;

use App\ItemList;
use App\Util;
use ORM;

class WPDBItemList implements ItemList
{

    protected $wpConfig;
    protected $columns = array(
        'itemId',
        'itemName',
        'itemPrice',
        'itemGroupId',
        'itemStock',
        'itemWeight',
    );
    protected $customFieldCache = array();

    public function __construct($wpConfig)
    {
        $this->wpConfig = $wpConfig;
        $this->initConfig();
    }

    public function initConfig()
    {
        ORM::configure(array(
            'connection_string' => "mysql:host={$this->wpConfig['DB_HOST']};dbname={$this->wpConfig['DB_NAME']}",
            'username' => $this->wpConfig['DB_USER'],
            'password' => $this->wpConfig['DB_PASSWORD'],
        ));
        ORM::configure('id_column_overrides', array(
            'wp_posts' => 'ID',
            'wp_postmeta' => 'meta_id',
        ));
        ORM::configure('driver_options',
            array(\PDO::MYSQL_ATTR_INIT_COMMAND => "SET NAMES {$this->wpConfig['DB_CHARSET']}")
        );
    }

    public function findOne($id)
    {
        $post = $this->getPost($id);
        if ($post === false) {
            return false;
        }
        $data = $this->post2columns($post);
        return $data;
    }

    public function getPost($id)
    {
        // Clear custom field cache.
        $this->customFieldCache = null;

        $orm = ORM::for_table('wp_posts')
                  ->table_alias('p')
                  ->select('p.*')
                  ->where('p.post_status', 'publish');

        list($type, $typeValue) = explode(':', $this->wpConfig['dataSource']);
        if (Util::is_empty($type) || Util::is_empty($typeValue)) {
            throw new \RuntimeException('不正なdataSourceです。');
        }
        if ($type !== 'post_type' && $type !== 'category') {
            throw new \RuntimeException("不正なdataSource type[{$type}]です。");
        }
        $typeValues = array_map("trim", explode(',', $typeValue));
        if (count($typeValues) > 0 && strlen($typeValues[0]) === 0) {
            throw new \RuntimeException("不正なdataSource typeValue[{$typeValues[0]}]です。");
        }

        if ($type === 'post_type') {
            if (count($typeValues) === 1) {
                $orm->where("p.$type", $typeValues[0]);
            } else {
                $orm->where_in("p.$type", $typeValues);
            }
        } elseif ($type === 'category') {
            $orm->where('p.post_type', 'post');
            $orm->join('wp_term_relationships', array('p.ID', '=', 'tr.object_id'), 'tr');
            $orm->join('wp_term_taxonomy', array('tr.term_taxonomy_id', '=', 'tt.term_taxonomy_id'), 'tt');
            $orm->where('tt.taxonomy', 'category');
            if (count($typeValues) === 1) {
                $orm->where('tt.term_id', $typeValues[0]);
            } else {
                $orm->where_in('tt.term_id', $typeValues);
            }
        }

        list($type, $name) = $this->getColumnConfig('itemId');
        if ($type === 'post') {
            $orm->where('p.ID', $id);
        } elseif ($type === 'custom') {
            $orm->join('wp_postmeta', array('p.ID', '=', 'pm.post_id'), 'pm');
            $orm->where(array('pm.meta_key' => $name, 'pm.meta_value' => $id));
        }

        $orm->order_by_asc('p.ID');

        $post = $orm->find_one();

        return $post;
    }

    public function getColumnConfig($name)
    {
        if (!array_key_exists($name, $this->wpConfig)) {
            throw new \RuntimeException("不正なカラム名[{$name}]です。");
        }
        $configStr = $this->wpConfig[$name];
        list($type, $value) = explode(':', $configStr);
        if (Util::is_empty($type) || Util::is_empty($value)) {
            throw new \RuntimeException("不正なカラム設定です。");
        }
        if ($type !== 'post' && $type !== 'custom') {
            throw new \RuntimeException("不正なカラムタイプ[{$type}]です。");
        }
        return array($type, $value);
    }

    public function post2columns($post)
    {
        $data = array();
        foreach ($this->columns as $column) {
            list($type, $name) = $this->getColumnConfig($column);
            if ($type === 'post') {
                $data[] = $post->$name;
            } elseif ($type === 'custom') {
                $data[] = $this->getPostCustomValue($name, $post->ID);
            } else {
                $data[] = null;
            }
        }
        return $data;
    }

    public function getPostCustomValue($name, $id)
    {
        $value = null;
        $customFields = $this->getPostCustomFields($id);
        if ($customFields) {
            // NOTE: Returns the first record's value.
            foreach ($customFields as $customField) {
                if ($customField->meta_key === $name) {
                    $value = $customField->meta_value;
                    break;
                }
            }
        }
        return $value;
    }

    public function getPostCustomFields($id)
    {
        $customFields = null;
        if ($this->customFieldCache) {
            $customFields = $this->customFieldCache;
        } else {
            $customFields = ORM::for_table('wp_postmeta')
                               ->where('post_id', $id)
                               ->order_by_asc('meta_id')
                               ->find_many();
            $this->customFieldCache = $customFields;
        }
        return $customFields;
    }

    public function update($props)
    {
        $id = $props['id'];
        $post = $this->getPost($id);
        if ($post === false) {
            return false;
        }
        list($type, $name) = $this->getColumnConfig('itemStock');
        if ($type !== 'custom') {
            throw new \RuntimeException('在庫数はカスタムフィールドのみサポートします。');
        }
        $stock = $props['stock'];
        $this->updatePostMeta($post->ID, $name, $stock);
        return true;
    }

    // NOTE: 実際は stock のみ更新する

    public function updatePostMeta($id, $name, $value)
    {
        $customFields = $this->getPostCustomFields($id);
        if ($customFields) {
            foreach ($customFields as $customField) {
                if ($customField->meta_key === $name) {
                    $customField->meta_value = $value;
                    $customField->save();
                    return;
                }
            }
        }
    }

    public function close()
    {
    }

    public function save()
    {
    }

}